function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import Button, { ButtonGroup } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/button.js";
import { ContainerHeader, HeaderSection, LayoutManager, NavigationProvider } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/navigation-next.js";
import Page from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/page.js";
import PageHeader from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/page-header.js";
import { FlexContainer } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/right-side-panel.js";
import Tooltip from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/tooltip.js";
import styled from "/apps/websight-atlaskit-esm/web-resources/styled-components.js";
import Breadcrumbs from "/apps/websight-admin/web-resources/Breadcrumbs.js";
import { PageContentContainer } from "/apps/websight-admin/web-resources/Containers.js";
import GlobalNavigation from "/apps/websight-admin/web-resources/GlobalNavigation.js";
import { AvatarIcon } from "/apps/websight-admin/web-resources/Icons.js";
import ListenForKeyboardShortcut from "/apps/websight-admin/web-resources/ListenForKeyboardShortcut.js";
import { getUrlHashValue, setUrlHashValue } from "/apps/websight-admin/web-resources/services/SearchParamsService.js";
import Footer from "/apps/websight-admin/web-resources/Footer.js";
import { AUTH_CONTEXT_UPDATED } from "/apps/websight-rest-atlaskit-client/web-resources/js/RestClient.js";
import CodeEditor from "./components/CodeEditor.js";
import Helper from "./components/Helper.js";
import SaveAsModal from "./components/SaveAsModal.js";
import ScriptsMenu from "./components/ScriptsMenu.js";
import ScriptService from "./services/ScriptService.js";
import { GROOVY_CONSOLE_ROOT_PATH } from "./utils/GroovyConsoleConstants.js";
const PageContainer = styled.div`
    overflow-x: auto;
    width: 100%;
`;

const NavigationHeader = () => /*#__PURE__*/React.createElement(HeaderSection, null, ({
  css
}) => /*#__PURE__*/React.createElement("div", {
  style: { ...css,
    paddingBottom: 20
  }
}, /*#__PURE__*/React.createElement(ContainerHeader, {
  before: () => /*#__PURE__*/React.createElement(AvatarIcon, {
    className: "material-icons"
  }, "code"),
  href: GROOVY_CONSOLE_ROOT_PATH,
  text: "Groovy Console"
})));

export default class GroovyConsole extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      isExecuting: false,
      executionResults: ScriptService.emptyExecutionResults,
      recentScripts: [],
      script: ScriptService.emptyScript
    };
    this.updateScript = this.updateScript.bind(this);
    this.saveScript = this.saveScript.bind(this);
    this.executeScript = this.executeScript.bind(this);
    this.onHashChange = this.onHashChange.bind(this);
    this.onScriptChange = this.onScriptChange.bind(this);
    this.onScriptDelete = this.onScriptDelete.bind(this);
    this.preventLeavingThePage = this.preventLeavingThePage.bind(this);
  }

  componentDidMount() {
    this.getRecentScripts();
    this.openScriptFromUrlHash();
    window.addEventListener('beforeunload', this.preventLeavingThePage);
    window.addEventListener('hashchange', this.onHashChange);
    window.addEventListener(AUTH_CONTEXT_UPDATED, () => {
      this.getRecentScripts();
      this.openScriptFromUrlHash();
    });
  }

  componentWillUnmount() {
    window.removeEventListener('beforeunload', this.preventLeavingThePage);
    window.removeEventListener('hashchange', this.onHashChange);
  }

  componentWillUpdate(nextProps, nextState) {
    const {
      script
    } = this.state;

    if (nextState.script.name !== script.name && nextState.script.groovy !== script.groovy) {
      this.setState({
        executionResults: ScriptService.emptyExecutionResults
      });
    }
  }

  onHashChange() {
    this.openScriptFromUrlHash();
  }

  openScriptFromUrlHash() {
    const {
      script
    } = this.state;
    const scriptPath = getUrlHashValue();

    if (scriptPath && script.path !== scriptPath) {
      ScriptService.listScripts(data => {
        const result = Object.entries(data.items).map(([key, value]) => {
          if (value.path === scriptPath) {
            return data.items[key];
          }
        }).filter(item => item !== undefined);

        if (result.length > 0 && result[0].path) {
          this.scriptsMenu.onScriptSelect(result[0]);
        }
      });
    }
  }

  preventLeavingThePage(e) {
    const {
      script,
      isExecuting
    } = this.state;

    if (script.changed || isExecuting) {
      e.returnValue = '';
      e.preventDefault();
    } else {
      delete e['returnValue'];
    }
  }

  getRecentScripts() {
    ScriptService.getRecentScripts(scripts => this.setState({
      recentScripts: scripts || []
    }));
  }

  updateScript(scriptData) {
    this.setState(prevState => ({
      script: { ...prevState.script,
        ...scriptData
      }
    }));
  }

  saveScript() {
    ScriptService.saveScript(this.state.script, () => {
      this.onScriptChange({
        path: this.state.script.path
      });
      this.getRecentScripts();
    });
  }

  executeScript() {
    const {
      script,
      isExecuting
    } = this.state;

    if (script.groovy && !isExecuting) {
      this.setState({
        isExecuting: true
      }, ScriptService.executeScript(script, executionResults => this.setState({
        executionResults,
        isExecuting: false
      }, this.getRecentScripts), () => this.setState({
        isExecuting: false
      })));
    }
  }

  onScriptChange(scriptData, getScriptCode = false) {
    setUrlHashValue(scriptData.path);

    if (getScriptCode) {
      ScriptService.getScriptCode(scriptData, data => {
        this.updateScript({ ...scriptData,
          groovy: data,
          changed: false
        });
      });
    } else {
      this.updateScript({ ...scriptData,
        changed: false
      });
    }
  }

  onScriptDelete() {
    this.getRecentScripts();
  }

  shouldComponentUpdate(prevState, state) {
    return JSON.stringify(this.state) !== JSON.stringify(state);
  }

  render() {
    const {
      script,
      recentScripts,
      isExecuting
    } = this.state;
    const actions = /*#__PURE__*/React.createElement(ButtonGroup, null, /*#__PURE__*/React.createElement(Tooltip, {
      delay: 0,
      content: "Alt+R"
    }, /*#__PURE__*/React.createElement(Button, {
      isDisabled: !script.groovy || isExecuting,
      isLoading: isExecuting,
      appearance: "primary",
      onClick: this.executeScript,
      iconBefore: /*#__PURE__*/React.createElement("i", {
        className: "material-icons"
      }, "play_arrow")
    }, "Run")), script.name && /*#__PURE__*/React.createElement(Button, {
      isDisabled: !script.groovy || !script.changed,
      onClick: this.saveScript
    }, "Save"), /*#__PURE__*/React.createElement(Button, {
      isDisabled: !script.groovy,
      onClick: () => this.saveAsModal.open()
    }, "Save as..."), /*#__PURE__*/React.createElement(SaveAsModal, {
      ref: element => this.saveAsModal = element,
      script: this.state.script,
      onSuccess: scriptData => {
        this.onScriptChange(scriptData);
        this.getRecentScripts();
      }
    }));
    const keyShortcuts = [{
      key: 'AltLeft+R',
      keyCodes: ['AltLeft', 'KeyR'],
      callback: this.executeScript
    }, {
      key: 'AltRight+R',
      keyCodes: ['AltRight', 'KeyR'],
      callback: this.executeScript
    }];
    return /*#__PURE__*/React.createElement(React.Fragment, null, keyShortcuts.map(shortcut => /*#__PURE__*/React.createElement(ListenForKeyboardShortcut, _extends({
      key: shortcut.key
    }, shortcut))), /*#__PURE__*/React.createElement(NavigationProvider, null, /*#__PURE__*/React.createElement(LayoutManager, {
      key: "layout-manager",
      globalNavigation: GlobalNavigation,
      productNavigation: () => null,
      containerNavigation: () => /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(NavigationHeader, null), /*#__PURE__*/React.createElement(ScriptsMenu, {
        key: "leftNavigation",
        ref: element => this.scriptsMenu = element,
        script: this.state.script,
        onScriptChange: this.onScriptChange,
        onScriptDelete: this.onScriptDelete,
        onHelperClick: () => this.helper.open(),
        recentScripts: recentScripts
      })),
      experimental_horizontalGlobalNav: true
    }, /*#__PURE__*/React.createElement(FlexContainer, {
      id: "groovy-container"
    }, /*#__PURE__*/React.createElement(PageContainer, null, /*#__PURE__*/React.createElement(Page, null, /*#__PURE__*/React.createElement(PageContentContainer, null, /*#__PURE__*/React.createElement(PageHeader, {
      actions: actions,
      breadcrumbs: /*#__PURE__*/React.createElement(Breadcrumbs, {
        breadcrumbs: [{
          text: 'Groovy Console',
          path: GROOVY_CONSOLE_ROOT_PATH,
          reactPath: ''
        }]
      })
    }, "Script: ", script.name ? script.name : 'Not saved'), /*#__PURE__*/React.createElement(CodeEditor, {
      ref: element => this.scriptEditor = element,
      executionResults: this.state.executionResults,
      isExecuting: this.state.isExecuting,
      script: this.state.script,
      onScriptChange: this.updateScript
    })), /*#__PURE__*/React.createElement(Footer, null))), /*#__PURE__*/React.createElement(Helper, {
      ref: element => this.helper = element,
      onAnimationFinished: () => this.scriptEditor.refreshEditors()
    })))));
  }

}