import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import { GroupHeading, Item, ItemPrimitive } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/navigation-next.js";
import { getUrlHashValue } from "/apps/websight-admin/web-resources/services/SearchParamsService.js";
import { colors } from "/apps/websight-admin/web-resources/theme.js";
import { getAuthorizableTypeFromHash, getIconNameForAuthorizableType } from "../utils/AuthorizableUtil.js";
import { toRelative } from "../utils/DateFormatter.js";
const actionTypeToLabel = {
  create: 'Created',
  read: 'Viewed',
  edit: 'Edited',
  delete: 'Deleted',
  disable: 'Disabled',
  enable: 'Enabled'
};
export default class RecentActions extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      authorizableType: getAuthorizableTypeFromHash(getUrlHashValue())
    };
    this.onHashChange = this.onHashChange.bind(this);
    this.updateAuthorizableType = this.updateAuthorizableType.bind(this);
  }

  componentDidMount() {
    window.addEventListener('hashchange', this.onHashChange);
  }

  componentWillUnmount() {
    window.removeEventListener('hashchange', this.onHashChange);
  }

  onHashChange() {
    this.updateAuthorizableType();
  }

  updateAuthorizableType() {
    this.setState({
      authorizableType: getAuthorizableTypeFromHash(getUrlHashValue())
    });
  }

  renderAuthorizables(authorizables = [], icon, type) {
    return authorizables.map(authorizable => authorizable.actionType === 'delete' ? /*#__PURE__*/React.createElement(ItemPrimitive, {
      before: () => /*#__PURE__*/React.createElement("i", {
        className: "material-icons",
        style: {
          color: colors.grey
        }
      }, icon),
      key: authorizable.id,
      text: /*#__PURE__*/React.createElement("div", {
        style: {
          color: colors.grey
        }
      }, authorizable.displayName),
      subText: authorizable.actionType ? this.prepareSubText(authorizable.actionType, authorizable.time) : undefined,
      styles: styles => ({ ...styles,
        itemBase: { ...styles.itemBase,
          cursor: 'default'
        }
      })
    }) : /*#__PURE__*/React.createElement(Item, {
      before: () => /*#__PURE__*/React.createElement("i", {
        className: "material-icons"
      }, icon),
      key: authorizable.id,
      text: authorizable.disabled ? /*#__PURE__*/React.createElement("del", null, authorizable.displayName) : authorizable.displayName,
      href: `#/${type}s/${encodeURIComponent(authorizable.id)}`,
      subText: authorizable.actionType ? this.prepareSubText(authorizable.actionType, authorizable.time) : undefined
    }));
  }

  prepareSubText(actionType, time) {
    const relativeTime = toRelative(time);
    return `${actionTypeToLabel[actionType]} ${relativeTime ? relativeTime : ''}`;
  }

  render() {
    const {
      recentAuthorizables
    } = this.props;
    const {
      authorizableType
    } = this.state;
    const authorizablesToRender = this.renderAuthorizables(recentAuthorizables[authorizableType], getIconNameForAuthorizableType(authorizableType), authorizableType);
    return authorizablesToRender.length > 0 && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(GroupHeading, null, "Recently visited"), authorizablesToRender);
  }

}