import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import Button from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/button.js";
import TableTree, { Header, Headers, Rows } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/table-tree.js";
import styled from "/apps/websight-atlaskit-esm/web-resources/styled-components.js";
import { NoTableDataContainer, TableTopAlignedContainer } from "/apps/websight-admin/web-resources/Containers.js";
import { colors } from "/apps/websight-admin/web-resources/theme.js";
import { LoadingWrapper } from "/apps/websight-admin/web-resources/Wrappers.js";
import PermissionsService from "../../services/PermissionsService.js";
import CreateAcEntryModal from "./AccessControlEntryModal.js";
import AceRow from "./AceRow.js";
import InheritedEntries from "./InheritedEntries.js";
const addEntryButtonStyle = {
  margin: '0 20px 0 0'
};
const PermissionsSectionContainer = styled.div`
    display: block;
    margin: 10px 0 0;
    width: 100%;
    margin-bottom: 40px;
`;
const StickyContentContainer = styled.div`
    background: ${colors.white}
    padding-top: 10px;
    position: sticky;
    top: 56px;
    z-index: 1;
`;
const AddEntryButtonContainer = styled.div`
    align-items: flex-end;
    display: flex;
    flex-direction: column;
    justify-content: flex-end;
    width: 100%;
    margin-bottom: 20px;
`;

const groupEntries = entries => {
  const groupedEntries = {};
  (entries || []).forEach(entry => {
    const {
      authorizableId
    } = entry;

    if (authorizableId in groupedEntries) {
      groupedEntries[authorizableId] = [...groupedEntries[authorizableId], entry];
    } else {
      groupedEntries[authorizableId] = [entry];
    }
  });
  let entriesWithSections = [];
  Object.entries(groupedEntries).forEach(([authorizableId, entriesArray]) => {
    entriesWithSections = [...entriesWithSections, {
      sectionHeader: authorizableId,
      sectionHierarchy: entriesArray.length ? entriesArray[0].membershipHierarchy || [] : []
    }];
    (entriesArray || []).forEach(entry => {
      entriesWithSections = [...entriesWithSections, entry];
    });
  });
  return entriesWithSections;
};

export default class AdvancedPermissions extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      isLoadingEntries: false,
      entries: [],
      inheritedEntries: []
    };
    this.loadEntries = this.loadEntries.bind(this);
  }

  componentDidMount() {
    this.loadEntries();
  }

  loadEntries() {
    const {
      authorizableId
    } = this.props;
    this.setState({
      isLoadingEntries: true
    }, () => PermissionsService.findAclEntries({
      authorizableId: authorizableId
    }, data => {
      this.setState({
        entries: data.entries,
        inheritedEntries: groupEntries(data.inheritedEntries),
        isLoadingEntries: false
      });
    }, () => {
      this.setState({
        isLoadingEntries: false
      });
    }));
  }

  render() {
    const {
      isAdmin,
      authorizableId,
      displayName,
      type,
      onAuthorizableAction
    } = this.props;
    const {
      inheritedEntries,
      isLoadingEntries
    } = this.state;
    const tableTreeHeaders = /*#__PURE__*/React.createElement(Headers, null, /*#__PURE__*/React.createElement(Header, {
      width: 'calc(90% - 700px)'
    }, "Path"), /*#__PURE__*/React.createElement(Header, {
      width: '110px'
    }, "Permission"), /*#__PURE__*/React.createElement(Header, {
      width: '260px'
    }, "Privileges"), /*#__PURE__*/React.createElement(Header, {
      width: 'calc(200px + 10%)'
    }, "Restrictions"), /*#__PURE__*/React.createElement(Header, {
      width: '130px'
    }, "Actions"));
    return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(PermissionsSectionContainer, null, /*#__PURE__*/React.createElement(TableTree, {
      key: "entries"
    }, /*#__PURE__*/React.createElement(StickyContentContainer, null, /*#__PURE__*/React.createElement("div", {
      style: {
        display: 'flex'
      }
    }, /*#__PURE__*/React.createElement(AddEntryButtonContainer, null, !isAdmin && /*#__PURE__*/React.createElement(Button, {
      onClick: () => this.createAcEntryModal.open(),
      style: addEntryButtonStyle
    }, "Add ACE"), isAdmin && /*#__PURE__*/React.createElement("i", {
      style: {
        marginRight: '20px'
      }
    }, "Admin user has full access to the repository"))), tableTreeHeaders), /*#__PURE__*/React.createElement(LoadingWrapper, {
      isLoading: this.state.isLoadingEntries,
      spinnerSize: "large"
    }, /*#__PURE__*/React.createElement(TableTopAlignedContainer, null, /*#__PURE__*/React.createElement(Rows, {
      items: this.state.entries,
      render: row => /*#__PURE__*/React.createElement(AceRow, {
        authorizableId: this.props.authorizableId,
        entry: row,
        onUpdate: () => {
          this.loadEntries();

          if (onAuthorizableAction) {
            onAuthorizableAction('edit', {
              id: authorizableId,
              displayName,
              type
            });
          }
        }
      })
    })), !this.state.entries.length && !this.state.isLoadingEntries && /*#__PURE__*/React.createElement(NoTableDataContainer, null, /*#__PURE__*/React.createElement("i", null, "No Access Control Entries.")))), /*#__PURE__*/React.createElement(InheritedEntries, {
      inheritedEntries: inheritedEntries,
      isLoadingEntries: isLoadingEntries
    }), /*#__PURE__*/React.createElement(CreateAcEntryModal, {
      ref: element => this.createAcEntryModal = element,
      authorizableId: this.props.authorizableId,
      onSaveSuccess: () => {
        this.loadEntries();

        if (onAuthorizableAction) {
          onAuthorizableAction('edit', {
            id: authorizableId,
            displayName,
            type
          });
        }
      }
    })));
  }

}